package com.example.db_sync;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.app.ProgressDialog;
import android.content.ContentResolver;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.database.Cursor;
import android.net.Uri;
import android.os.Bundle;
import android.support.v4.app.LoaderManager;
import android.support.v4.content.CursorLoader;
import android.support.v4.content.Loader;
import android.support.v4.widget.SimpleCursorAdapter;
import android.support.v7.app.AppCompatActivity;
import android.text.TextUtils;
import android.util.Log;
import android.view.ContextMenu;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.ListView;
import android.widget.Toast;

import com.example.db_sync.contentprovider.dB_contentprovider;
import com.example.db_sync.database.tbl_udaje;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.loopj.android.http.AsyncHttpClient;
import com.loopj.android.http.AsyncHttpResponseHandler;
import com.loopj.android.http.RequestParams;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.HashMap;

public class MainActivity extends AppCompatActivity implements LoaderManager.LoaderCallbacks<Cursor> {
    private ListView m_zoznam;

    // obsah kontextoveho menu
    static final private int CHANGE_ITEM = Menu.FIRST;
    static final private int REMOVE_ITEM = Menu.FIRST + 1;
    static final private int REMOVE_ALL_ITEMS = Menu.FIRST + 2;

    // kontext MainActivity
    private static Context context;

    private static final int CHANGE_LIST_ITEM=0;

    // SimpleCursorAdapter pre naplnenie ListView udajmi z kurzora
    private SimpleCursorAdapter m_sca;

    // ContentResolver
    private ContentResolver m_cr;

    // Progress Dialog objekt
    ProgressDialog m_progress;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        final EditText m_text; // form pre vlozenie textu
        Button m_tlacidlo; // tlacidlo pre ulozenie textu do tabulky

        // kontext MainActivity
        context=this;

        // spristupnenie UI widgetov definovanych v XML layoute activity_main.xml
        m_text = (EditText)findViewById(R.id.data);
        m_tlacidlo = (Button)findViewById(R.id.tlacidlo);
        m_zoznam = (ListView)findViewById(R.id.zoznam);

        // kontextove menu pre ListView
        registerForContextMenu(m_zoznam);

        // ziskanie ContentResolveru patriaceho ku kontextu aktivity
        // ContentResolver ziada o vykonanie uloh ContentProvider s danym URI
        m_cr = getContentResolver();

        // progres dialog
        m_progress=new ProgressDialog(this);
        m_progress.setMessage("Synchronizujem externu MySQL dB s internou SQLite dB...");
        m_progress.setCancelable(false);

        // intent objekt pre spustenie BroadcastReceivera MyReceiver
        Intent my_intent = new Intent(context, MyReceiver.class);
        // PendingIntent objekt - token odovzdany AlarmManageru, pomocou ktoreho moze AlarmManager
        // spustat kod BroadcastReceivera MyReceiver s rovnakymi pravami ako ma MainActivity
        PendingIntent pendingIntent = PendingIntent.getBroadcast(context, 0, my_intent, PendingIntent.FLAG_UPDATE_CURRENT);
        // AlarmManager objekt
        AlarmManager alarmManager = (AlarmManager) context.getSystemService(Context.ALARM_SERVICE);
        // AlarmManager spusti BroadcastReceiver po 1 minute (1 * 60 * 1000) a nasledne
        // spusta BroadcastReceiver kazdych 5 minut (5 * 60 * 1000)
        // BroadcastReceiver spusta MyService v pripade zmeny MySQL dB
        alarmManager.setRepeating(AlarmManager.RTC_WAKEUP,
                                  Calendar.getInstance().getTimeInMillis() + 1 * 60 * 1000,
                                  5 * 60 * 1000,
                                  pendingIntent);

        // reakcia tlacidla m_tlacidlo na kliknutie prstom
        m_tlacidlo.setOnClickListener(new View.OnClickListener() {
            public void onClick(View view) {
                if (TextUtils.isEmpty(m_text.getText().toString())) {
                    makeToast(getResources().getString(R.string.nezadany_udaj));
                } else {
                    // novy riadok stlpec/hodnota
                    ContentValues row = new ContentValues();
                    // priradenie udajov novemu riadku
                    row.put(tbl_udaje.COLUMN_DATA, m_text.getText().toString());

                    SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm:ss");
                    Calendar rightNow = Calendar.getInstance();
                    Date curDate = rightNow.getTime();
                    row.put(tbl_udaje.COLUMN_DATETIME, sdf.format(curDate));

                    Log.d("datetime: ", sdf.format(curDate));

                    // novo vlozeny riadok nie je zosynchronizovany
                    row.put(tbl_udaje.COLUMN_SYNCED, 0);
                    // novo vlozeny riadok nie je oznaceny na zmazanie
                    row.put(tbl_udaje.COLUMN_TOBEDELETED, 0);

                    // pridanie riadku do tabulky
                    m_cr.insert(dB_contentprovider.CONTENT_URI, row);

                    // vymazanie obsahu UI widgetu m_text
                    m_text.setText("");
                }
            }
        });

        // naplnenie zoznamu poloziek udajmi z internej SQLite dB
        napln_zoznam_poloziek();
    }

    // naplnenie zoznamu poloziek udajmi z internej dB
    private void napln_zoznam_poloziek() {
        // Loader pre nacitanie udajov - pripojenie k existujucemu,
        // alebo start noveho Loadera + nastartovanie background framework
        getSupportLoaderManager().initLoader(0, null, this);

        // String array pre SCA
        String[] from = new String[] { tbl_udaje.COLUMN_DATA,
                                       tbl_udaje.COLUMN_DATETIME,
                                       tbl_udaje.COLUMN_SYNCED,
                                       tbl_udaje.COLUMN_TOBEDELETED };
        // integer array pre SCA
        int[] to = new int[] { R.id.d, R.id.dt, R.id.s, R.id.tbd };

        // vytvorenie SimpleCursorAdaptera pre spravu a zobrazenie udajov ListView
        // Context, Layout identificator, Cursor, String array, integer array, flag
        m_sca = new SimpleCursorAdapter(this, R.layout.polozka, null, from, to, 0);

        // nastavenie adaptera zodpovedneho za udaje (vratane ich View) daneho ListView
        m_zoznam.setAdapter(m_sca);
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // zahrnutie menu do ActionBaru
        getMenuInflater().inflate(R.menu.menu_main, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        // reakcia menu na kliknutie prstom
        if (id == R.id.action_synchronize) {

            m_progress.show();

            // synchronizacia internej SQLite dB podla externej MySQL dB
            syncSQLite_to_MySQL();
            // synchronizacia externej MySQL dB podla internej SQLite dB
            syncMySQL_to_SQLite();

            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    // synchronizacia internej SQLite dB podla externej MySQL dB
    public void syncSQLite_to_MySQL() {
        // AsycHttpClient objekt
        AsyncHttpClient client = new AsyncHttpClient();
        // HttpRequestParams objekt
        RequestParams params = new RequestParams();

        // HTTP spustenie skriptu get_tbd_rows.php - riadky oznacene na zmazanie
        client.post(getResources().getString(R.string.get_tbd_rowsURI), params, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(String ret) {
                // aktualizacia obsahu SQLite dB a MySQL dB na zaklade udajov nacitanych z MySQL dB
                delete_from_SQLitedB(ret);
                // zmazanie riadkov z MySQL dB po vymazani riadkov z SQLite dB
                delete_from_MySQLdB();
            }
            @Override
            public void onFailure(int statusCode, Throwable error, String content) {
                if (statusCode == 404) { makeToast("URI nenajdene!"); }
                else if (statusCode == 500) { makeToast("Chyba na strane servera!"); }
                     else { makeToast("Chyba pri vykonani skriptu!"); }
            }
        });
        // HTTP spustenie skriptu getunsyncedrows.php - nezosynchronizovane riadky
        client.post(getResources().getString(R.string.getunsyncedrowsURI), params, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(String ret) {
                // aktualizacia obsahu SQLite dB na zaklade udajov nacitanych z MySQL dB
                update_SQLitedB(ret);
            }

            @Override
            public void onFailure(int statusCode, Throwable error, String content) {
                if (statusCode == 404) {
                    makeToast("URI nenajdene!");
                } else if (statusCode == 500) {
                    makeToast("Chyba na strane servera!");
                } else {
                    makeToast("Chyba pri vykonani skriptu!");
                }
            }
        });
    }

    // zmazanie riadkov z SQLite dB na zaklade udajov nacitanych z MySQL dB
    public void delete_from_SQLitedB(String ret) {
        try {
            // naplnenie JSON pola udajmi vratenymi zo servera
            JSONArray arr = new JSONArray(ret);
            // ak sa v JSON poli nachadzaju udaje
            if (arr.length() != 0) {
                // pre vsetky udaje = pre vsetky riadky oznacene na zmazanie
                for (int i = 0; i < arr.length(); i++) {
                    // i-ty prvok pola
                    JSONObject obj=(JSONObject) arr.get(i);

                    // zmazanie konkretneho riadku z SQLite dB
                    m_cr.delete(dB_contentprovider.CONTENT_URI,
                                tbl_udaje.COLUMN_DATETIME + "=?",
                                new String[] { obj.get("dtid").toString() });
                }
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }
    // zmazanie riadkov z MySQL dB po vymazani riadkov z SQLite dB
    public void delete_from_MySQLdB() {
        // AsycHttpClient objekt
        AsyncHttpClient client = new AsyncHttpClient();
        // HttpRequestParams objekt
        RequestParams params = new RequestParams();

        // zmazanie riadkov oznacenych na zmazanie priamo v MySQL dB
        // odovzdanim "" skriptu delete_tbd_rows.php dojde k zmazaniu
        // tych riadkov, ktore boli oznacene na zmazanie priamo v MySQL dB
        // implicitnym nastavenim tobedeleted = 1
        params.put("row", "");

        // HTTP spustenie skriptu delete_tbd_rows.php
        client.post(getResources().getString(R.string.delete_tbd_rowsURI), params, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(String ret) {

            }

            @Override
            public void onFailure(int statusCode, Throwable error, String content) {
                if (statusCode == 404) { makeToast("URI nenajdene!"); }
                else if (statusCode == 500) { makeToast("Chyba na strane servera!"); }
                     else { makeToast("Chyba pri vykonani skriptu!"); }
            }
        });
    }

    // aktualizacia obsahu SQLite dB na zaklade udajov nacitanych z MySQL dB
    public void update_SQLitedB(String ret) {
        // zoznam zosynchronizovanych riadkov
        ArrayList<HashMap<String, String>> synchronized_rows = new ArrayList<>();
        // GSON objekt
        Gson gson = new GsonBuilder().create();

        String[] projection = { tbl_udaje.COLUMN_ID,
                                tbl_udaje.COLUMN_DATETIME,
                                tbl_udaje.COLUMN_DATA,
                                tbl_udaje.COLUMN_SYNCED,
                                tbl_udaje.COLUMN_TOBEDELETED };

        try {
            // naplnenie JSON pola udajmi vratenymi zo servera
            JSONArray arr = new JSONArray(ret);
            // ak sa v JSON poli nachadzaju udaje
            if (arr.length() != 0) {
                // pre vsetky udaje = pre vsetky nezosynchronizovane riadky tabulky
                for (int i = 0; i < arr.length(); i++) {
                    // i-ty prvok pola
                    JSONObject obj=(JSONObject) arr.get(i);

                    // novy riadok stlpec/hodnota
                    ContentValues row = new ContentValues();
                    // priradenie udajov novemu riadku
                    row.put(tbl_udaje.COLUMN_DATETIME, obj.get("dtid").toString());
                    row.put(tbl_udaje.COLUMN_DATA, obj.get("data").toString());
                    // novy riadok je uz zosynchronizovany
                    row.put(tbl_udaje.COLUMN_SYNCED, 1);
                    // novy riadok nie je oznaceny na zmazanie
                    row.put(tbl_udaje.COLUMN_TOBEDELETED, 0);

                    // kontrola existencie riadku v SQLite dB
                    Cursor m_cursor = m_cr.query(dB_contentprovider.CONTENT_URI,
                            projection,
                            tbl_udaje.COLUMN_DATETIME + "=?",
                            new String[]{obj.get("dtid").toString()},
                            null);
                    m_cursor.moveToFirst();

                    if (m_cursor.getCount() > 0) { // riadok existuje => aktualizacia riadku
                        // aktualizacia konkretneho riadku v SQLite dB
                        Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                  + m_cursor.getString(m_cursor.getColumnIndex("_id")));
                        m_cr.update(uri, row, null, null);
                    } else { // riadok neexistuje => pridanie riadku
                        // pridanie noveho riadku do tabulky
                        m_cr.insert(dB_contentprovider.CONTENT_URI, row);
                    }

                    m_cursor.close();

                    // postupne naplnenie zoznamu synchronizovanych riadkov
                    HashMap<String, String> map = new HashMap<>();
                    map.put("id", obj.get("id").toString());
                    map.put("synced", "1");
                    synchronized_rows.add(map);
                }

                // po nacitani nezosynchronizovanych riadkov z MySQL dB je potrebne
                // riadky v MySQL dB oznacit statusom synced = 1

                // AsycHttpClient objekt
                AsyncHttpClient client = new AsyncHttpClient();
                // HttpRequestParams objekt
                RequestParams params = new RequestParams();

                // odoslanie zoznamu zosynchronizovanych riadkov PHP skriptu
                params.put("synced_status", gson.toJson(synchronized_rows));

                // HTTP spustenie skriptu changesyncedstatus.php
                client.post(getResources().getString(R.string.changesyncedstatusURI), params, new AsyncHttpResponseHandler() {
                    @Override
                    public void onSuccess(String ret) {
                        makeToast("MySQL dB bola aktualizovana");
                    }

                    @Override
                    public void onFailure(int statusCode, Throwable error, String content) {
                        if (statusCode == 404) { makeToast("URI nenajdene!"); }
                        else if (statusCode == 500) { makeToast("Chyba na strane servera!"); }
                             else { makeToast("Chyba pri vykonani skriptu!"); }
                    }
                });
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
    }

    // synchronizacia externej MySQL dB podla internej SQLite dB
    public void syncMySQL_to_SQLite() {
        // zoznam riadkov urcenych na zmazanie
        ArrayList<HashMap<String, String>> tobedeleted_rows = new ArrayList<>();
        // zoznam zosynchronizovanych riadkov
        ArrayList<HashMap<String, String>> synchronized_rows = new ArrayList<>();
        // GSON objekt
        Gson gson = new GsonBuilder().create();

        // kurzor na riadky internej SQLite dB
        String[] projection = { tbl_udaje.COLUMN_ID,
                                tbl_udaje.COLUMN_DATETIME,
                                tbl_udaje.COLUMN_DATA,
                                tbl_udaje.COLUMN_SYNCED,
                                tbl_udaje.COLUMN_TOBEDELETED };

        // 1. zmazanie riadkov z SQLite dB oznacenych na zmazanie
        Cursor m_cursor = m_cr.query(dB_contentprovider.CONTENT_URI,
                                     projection, null, null, null);
        m_cursor.moveToFirst();

        // pre vsetky riadky tabulky
        for (int i = 0; i < m_cursor.getCount(); i++) {
            // tobedeleted status
            int tbd = Integer.parseInt(m_cursor.getString(m_cursor.getColumnIndex("tobedeleted")));
            // ak je riadok oznaceny na zmazanie
            if (tbd == 1) {
                // postupne naplnenie zoznamu riadkov urcenych na zmazanie
                HashMap<String, String> map = new HashMap<>();
                map.put("dtid", m_cursor.getString(m_cursor.getColumnIndex("datetime")));
                tobedeleted_rows.add(map);

                // zmazanie riadku z SQLite dB
                Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                    + m_cursor.getString(m_cursor.getColumnIndex("_id")));
                m_cr.delete(uri,null,null);
            }

            m_cursor.moveToNext();
        }

        m_cursor.close();

        // zmazanie riadkov z MySQL dB po vymazani riadkov z SQLite dB
        delete_from_MySQLdB(gson.toJson(tobedeleted_rows));

        // 2. ziskanie udajov vsetkych nezosynchronizovanych riadkov
        m_cursor = m_cr.query(dB_contentprovider.CONTENT_URI,
                              projection, null, null, null);
        m_cursor.moveToFirst();

        // pre vsetky riadky tabulky
        for (int i = 0; i < m_cursor.getCount(); i++) {
            // synced status
            int synced = Integer.parseInt(m_cursor.getString(m_cursor.getColumnIndex("synced")));
            // ak nie je konkretny riadok zosynchronizovany
            if (synced != 1) {
                // zmena statusu riadku v SQLite dB na synced = 1
                Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                    + m_cursor.getString(m_cursor.getColumnIndex("_id")));
                ContentValues row = new ContentValues();
                // riadok je uz zosynchronizovany
                row.put(tbl_udaje.COLUMN_SYNCED, 1);
                m_cr.update(uri, row, null, null);

                // postupne naplnenie zoznamu synchronizovanych riadkov
                HashMap<String, String> map = new HashMap<>();
                map.put("dtid", m_cursor.getString(m_cursor.getColumnIndex("datetime")));
                map.put("data", m_cursor.getString(m_cursor.getColumnIndex("data")));
                map.put("synced", "1");
                // tobedeleted automaticky zostane 0
                synchronized_rows.add(map);
            }

            m_cursor.moveToNext();
        }

        m_cursor.close();

        // aktualizacia obsahu MySQL dB na zaklade udajov nacitanych z SQLite dB
        update_MySQLdB(gson.toJson(synchronized_rows));
    }

    // zmazanie riadkov z MySQL dB po vymazani riadkov z SQLite dB
    public void delete_from_MySQLdB(String m_jsonarray) {
        // AsycHttpClient objekt
        AsyncHttpClient client = new AsyncHttpClient();
        // HttpRequestParams objekt
        RequestParams params = new RequestParams();

        // odoslanie zoznamu riadkov urcenych na zmazanie PHP skriptu
        // odovzdanim zoznamu skriptu delete_tbd_rows.php dojde k zmazaniu
        // tych riadkov, ktore boli oznacene na zmazanie v SQLite dB
        // implicitnym nastavenim tobedeleted = 1
        params.put("row", m_jsonarray);
        Log.d("JSON: ", m_jsonarray);

        // HTTP spustenie skriptu delete_tbd_rows.php
        client.post(getResources().getString(R.string.delete_tbd_rowsURI), params, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(String ret) {

            }
            @Override
            public void onFailure(int statusCode, Throwable error, String content) {
                if (statusCode == 404) { makeToast("URI nenajdene!"); }
                else if (statusCode == 500) { makeToast("Chyba na strane servera!"); }
                else { makeToast("Chyba pri vykonani skriptu!"); }
            }
        });
    }

    // aktualizacia obsahu MySQL dB na zaklade udajov nacitanych z SQLite dB
    public void update_MySQLdB(String m_jsonarray) {
        // AsycHttpClient objekt
        AsyncHttpClient client = new AsyncHttpClient();
        // HttpRequestParams objekt
        RequestParams params = new RequestParams();

        // odoslanie zoznamu zosynchronizovanych riadkov PHP skriptu
        params.put("row", m_jsonarray);
        Log.d("JSON: ", m_jsonarray);

        // HTTP spustenie skriptu addunsyncedrows.php
        client.post(getResources().getString(R.string.addunsyncedrowsURI), params, new AsyncHttpResponseHandler() {
            @Override
            public void onSuccess(String ret) {
                m_progress.hide();
                makeToast("MySQL DB bola aktualizovana");
            }

            @Override
            public void onFailure(int statusCode, Throwable error, String content) {
                m_progress.hide();
                if (statusCode == 404) { makeToast("URI nenajdene!"); }
                else if (statusCode == 500) { makeToast("Chyba na strane servera!"); }
                     else { makeToast("Chyba pri vykonani skriptu!"); }
            }
        });
    }

    // kontextove menu pre ListView
    @Override
    public void onCreateContextMenu(ContextMenu menu,
                                    View v,
                                    ContextMenu.ContextMenuInfo menuInfo) {
        super.onCreateContextMenu(menu, v, menuInfo);
        menu.setHeaderTitle(R.string.obsah_tabulky);
        menu.add(0, CHANGE_ITEM, Menu.NONE, R.string.zmen_polozku);
        menu.add(0, REMOVE_ITEM, Menu.NONE, R.string.vymaz_polozku);
        menu.add(0, REMOVE_ALL_ITEMS, Menu.NONE, R.string.vymaz_vsetky_polozky);
    }

    // reakcia na vyber v ramci kontextoveho menu
    @Override
    public boolean onContextItemSelected(MenuItem item) {
        super.onContextItemSelected(item);
        switch (item.getItemId()) {
            case (CHANGE_ITEM): {
                // ziskanie doplnujucich udajov polozky kontextoveho menu
                AdapterView.AdapterContextMenuInfo menuInfo;
                menuInfo =(AdapterView.AdapterContextMenuInfo)item.getMenuInfo();

                String[] projection = { tbl_udaje.COLUMN_ID,
                                        tbl_udaje.COLUMN_DATA };
                // URI pre zmenu konkretneho riadku
                Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                    + menuInfo.id);
                Cursor m_cursor = m_cr.query(uri, projection, null, null, null);
                m_cursor.moveToFirst();

                // spustenie novej ChangeListItem aktivity
                Intent i = new Intent(context, ChangeListItem.class);

                i.putExtra("id", m_cursor.getString(m_cursor.getColumnIndex("_id")));
                i.putExtra("data", m_cursor.getString(m_cursor.getColumnIndex("data")));

                startActivityForResult(i, CHANGE_LIST_ITEM);

                m_cursor.close();

                return true;
            }
            case (REMOVE_ITEM): {
                // ziskanie doplnujucich udajov polozky kontextoveho menu
                AdapterView.AdapterContextMenuInfo menuInfo;
                menuInfo =(AdapterView.AdapterContextMenuInfo)item.getMenuInfo();
                // URI pre vymazanie konkretneho riadku
                Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                    + menuInfo.id);
                //m_cr.delete(uri, null, null);
                // namiesto zmazania iba nastavenie tobedeleted = 1
                ContentValues row = new ContentValues();
                // oznacenie riadku na zmazanie
                row.put(tbl_udaje.COLUMN_TOBEDELETED, 1);
                m_cr.update(uri, row, null, null);
                return true;
            }
            case (REMOVE_ALL_ITEMS): {
                // vymazanie vsetkych riadkov z tabulky dB
                //m_cr.delete(dB_contentprovider.CONTENT_URI, null, null);
                // namiesto zmazania iba nastavenie tobedeleted = 1
                ContentValues row = new ContentValues();
                // oznacenie riadkov na zmazanie
                row.put(tbl_udaje.COLUMN_TOBEDELETED, 1);
                m_cr.update(dB_contentprovider.CONTENT_URI, row, null, null);
                return true;
            }
        }
        return false;
    }

    // reakcia na vysledok ChangeListItem aktivity
    @Override
    protected void onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == CHANGE_LIST_ITEM) {
            if (resultCode != RESULT_OK) return;

            // vytvorenie riadku stlpec/hodnota
            ContentValues row = new ContentValues();
            // priradenie udajov riadku
            row.put(tbl_udaje.COLUMN_DATA, data.getStringExtra("result_data"));
            // riadok nie je zosynchronizovany
            row.put(tbl_udaje.COLUMN_SYNCED, 0);

            Uri uri = Uri.parse(dB_contentprovider.CONTENT_URI + "/"
                                + data.getStringExtra("result_id"));

            //Log.d("result_id",data.getStringExtra("result_id"));
            //Log.d("result_data",data.getStringExtra("result_data"));

            m_cr.update(uri, row, null, null);
        }
        super.onActivityResult(requestCode, resultCode, data);
    }

    // vytvorenie Loaderu v pripade ak je to potrebne
    // id = initLoader(id,...,...)
    @Override
    public Loader<Cursor> onCreateLoader(int id, Bundle args) {
        // stlpce tabulky
        String[] projection = { tbl_udaje.COLUMN_ID,
                                tbl_udaje.COLUMN_DATETIME,
                                tbl_udaje.COLUMN_DATA,
                                tbl_udaje.COLUMN_SYNCED,
                                tbl_udaje.COLUMN_TOBEDELETED };
        // vytvorenie Loaderu
        // Parent activity context, Table to query, Projection to return,
        // Selection clause, Selection arguments, Sort order
        // na zobrazenie udajov z Loaderu je potrebne:
        // 1. pouzit View implementujuci AdapterView
        // 2. spristupnit dany View cestou CursorAdapter
        return new CursorLoader(this, dB_contentprovider.CONTENT_URI, projection,
                null, null, null);
    }

    // Loader ukoncil nacitanie udajov (asynchronne na pozadi)
    // presun vysledkov query (cursor) do adapteru (CursorAdapter)
    // a nasledne automaticky View (ListView) re-display
    @Override
    public void onLoadFinished(Loader<Cursor> loader, Cursor cursor) {
        m_sca.swapCursor(cursor);
    }

    // Loader bol resetovany, udaje poskytovane Providerom sa zmenili
    // a kurzor uz nie je aktualny, po ukonceni resetu CursorLoader re-runs query
    @Override
    public void onLoaderReset(Loader<Cursor> loader) { m_sca.swapCursor(null); }

    // informacna sprava
    private void makeToast(String toasttext) {
        Toast.makeText(MainActivity.this, toasttext, Toast.LENGTH_LONG).show();
    }
}
