package com.example.db_sync.contentprovider;

import android.content.ContentProvider;
import android.content.ContentValues;
import android.content.UriMatcher;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteQueryBuilder;
import android.net.Uri;
import android.text.TextUtils;
import android.util.Log;

import com.example.db_sync.database.dB_helper;
import com.example.db_sync.database.tbl_udaje;

import java.util.Arrays;
import java.util.HashSet;

// trieda pre pristup k udajom ulozenym v dB
// ContentProvider sa navonok tvari ako dB (tabulka)
public class dB_contentprovider extends ContentProvider {
    // full URI ContentProvidera - musi byt jedinecny, standard je package name
    private static final String AUTHORITY = "com.example.db_sync.contentprovider";
    private static final String DATA_PATH = "udaje";
    // Content URI (Uniform Resource Identifier) = scheme:authority:path
    // scheme = content://
    // authority = app package name + provider name
    // path = table name - definuje provider podla struktury tabuliek
    public static final Uri CONTENT_URI = Uri.parse("content://" + AUTHORITY
            + "/" + DATA_PATH);
    // konstanty pre vyber formatu URI
    private static final int ALL_ROWS = 1;
    private static final int SINGLE_ROW = 2;
    // vyber formatu URI, o ktory ziada ContentResolver
    private static final UriMatcher myURIMatcher = new UriMatcher(UriMatcher.NO_MATCH);
    static {
        // nastavenie identifikatora ALL_ROWS pre URI vsetkych riadkov tabulky
        myURIMatcher.addURI(AUTHORITY, DATA_PATH, ALL_ROWS);
        // nastavenie identifikatora SINGLE_ROW pre URI konkretneho riadku tabulky
        myURIMatcher.addURI(AUTHORITY, DATA_PATH + "/#", SINGLE_ROW);
    }

    // praca s dB
    private SQLiteDatabase m_db;

    @Override
    public boolean onCreate() {
        // inicializacia datoveho zdroja = vytvorenie dB (tabulky)
        dB_helper m_helper = new dB_helper(getContext());

        m_db = m_helper.getWritableDatabase();

        return (m_db == null);
    }

    // definicia MIME typu, ktory identifikuje vystupne data ContentProvidera
    // pre vsetky polozky vnd.android.cursor.dir/vnd.app_package_name.data_type
    // pre konkretnu polozku vnd.android.cursor.item/vnd.app_package_name.data_type
    @Override
    public String getType(Uri uri) {
       switch (myURIMatcher.match(uri)) {
           case ALL_ROWS: return "vnd.android.cursor.dir/vnd.com.example.db_sync.udaje";
           case SINGLE_ROW: return "vnd.android.cursor.item/vnd.com.example.db_sync.udaje";
           default:
               throw new IllegalArgumentException("Nezname URI: " + uri);
       }
    }

    // URI ContentProvidera
    // projection - pozadovane stlpce
    // where - SQL vyraz, ? = udaje zo selection
    // selection - udaje pre nahradu ? vo where
    // poradie riadkov
    @Override
    public Cursor query(Uri uri, String[] projection, String selection,
                        String[] selectionArgs, String sortOrder) {

        // namiesto query() pouzije SQLiteQueryBuilder
        SQLiteQueryBuilder qb = new SQLiteQueryBuilder();

        // kontrola existencie stlpcov v tabulke
        checkColumns(projection);

        // vyber tabulky
        qb.setTables(tbl_udaje.DATABASE_TABLE);

        // ziadost o vyber vsetkych riadkov, resp. konkretneho riadku
        int uriType = myURIMatcher.match(uri);
        switch (uriType) {
            case ALL_ROWS:
                break;
            case SINGLE_ROW:
                // vyber konkretneho riadku
                qb.appendWhere(tbl_udaje.COLUMN_ID + "=" + uri.getLastPathSegment());
                break;
            default:
                throw new IllegalArgumentException("Nezname URI: " + uri);
        }

        // vykonanie query()
        Cursor cursor = qb.query(m_db, projection, selection,
                selectionArgs, null, null, sortOrder);

        // notifikacia ContentResolvera
        cursor.setNotificationUri(getContext().getContentResolver(), uri);

        return cursor;
    }

    @Override
    public Uri insert(Uri uri, ContentValues values) {
        long rowID;

        // ziadost o vyber vsetkych riadkov, resp. konkretneho riadku
        int uriType = myURIMatcher.match(uri);
        switch (uriType) {
            case ALL_ROWS:
                rowID = m_db.insert(tbl_udaje.DATABASE_TABLE, null, values);
                break;
            default:
                throw new IllegalArgumentException("Nezname URI: " + uri);
        }

        // notifikacia ContentResolvera
        getContext().getContentResolver().notifyChange(uri, null);

        Log.d("Novy riadok vlozeny: ", String.valueOf(rowID));

        return Uri.parse(DATA_PATH + "/" + rowID);
    }

    @Override
    public int delete(Uri uri, String selection, String[] selectionArgs) {
        int rowsDeleted;

        // ziadost o vyber vsetkych riadkov, resp. konkretneho riadku
        int uriType = myURIMatcher.match(uri);
        switch (uriType) {
            case ALL_ROWS:
                rowsDeleted = m_db.delete(tbl_udaje.DATABASE_TABLE, selection,
                        selectionArgs);
                Log.d("Vsetky riadky zmazane: ", String.valueOf(rowsDeleted));
                break;
            case SINGLE_ROW:
                String rowID = uri.getLastPathSegment();
                if (TextUtils.isEmpty(selection)) {
                    rowsDeleted = m_db.delete(tbl_udaje.DATABASE_TABLE,
                            tbl_udaje.COLUMN_ID + "=" + rowID,
                            null);
                } else {
                    rowsDeleted = m_db.delete(tbl_udaje.DATABASE_TABLE,
                            tbl_udaje.COLUMN_ID + "=" + rowID
                                    + " and " + selection,
                            selectionArgs);
                }
                Log.d("Riadok vymazany: ", String.valueOf(rowID));
                break;
            default:
                throw new IllegalArgumentException("Nezname URI: " + uri);
        }

        // notifikacia ContentResolvera
        getContext().getContentResolver().notifyChange(uri, null);

        return rowsDeleted;
    }

    @Override
    public int update(Uri uri, ContentValues values, String selection,
                      String[] selectionArgs) {
        int rowsUpdated;

        // ziadost o vyber vsetkych riadkov, resp. konkretneho riadku
        int uriType = myURIMatcher.match(uri);
        switch (uriType) {
            case ALL_ROWS:
                rowsUpdated = m_db.update(tbl_udaje.DATABASE_TABLE,
                        values,
                        selection,
                        selectionArgs);
                Log.d("Vsetky riadky zmenene: ", String.valueOf(rowsUpdated));
                break;
            case SINGLE_ROW:
                String rowID = uri.getLastPathSegment();
                if (TextUtils.isEmpty(selection)) {
                    rowsUpdated = m_db.update(tbl_udaje.DATABASE_TABLE,
                            values,
                            tbl_udaje.COLUMN_ID + "=" + rowID,
                            null);
                } else {
                    rowsUpdated = m_db.update(tbl_udaje.DATABASE_TABLE,
                            values,
                            tbl_udaje.COLUMN_ID + "=" + rowID
                                    + " and " + selection,
                            selectionArgs);
                }
                Log.d("Riadok zmeneny: ", String.valueOf(rowID));
                break;
            default:
                throw new IllegalArgumentException("Nezname URI: " + uri);
        }

        // notifikacia ContentResolvera
        getContext().getContentResolver().notifyChange(uri, null);

        return rowsUpdated;
    }

    // kontrola existencie stlpcov v tabulke
    private void checkColumns(String[] projection) {
        String[] available = { tbl_udaje.COLUMN_ID,
                               tbl_udaje.COLUMN_DATETIME,
                               tbl_udaje.COLUMN_DATA,
                               tbl_udaje.COLUMN_SYNCED,
                               tbl_udaje.COLUMN_TOBEDELETED };
        if (projection != null) {
            HashSet<String> requestedColumns = new HashSet<>(Arrays.asList(projection));
            HashSet<String> availableColumns = new HashSet<>(Arrays.asList(available));
            // kontrola existencie stlpcov
            if (!availableColumns.containsAll(requestedColumns)) {
                throw new IllegalArgumentException("Neexistujuci stlpec tabulky!");
            }
        }
    }
}